namespace eval ::Imago {

  variable imago_config_filename "SSI_PROFILES:imago/resource/scripts/imago-config.txt"

  proc display_warning { title warning } {

    puts ""
    puts "WARNING - ${title} - ${warning}"
    puts ""

    ::do_display_warning $title $warning

  }

  proc load_config {} {
    variable imago_config_filename

    if {[catch {open $imago_config_filename r} fp]} {
      return [dict create]
    }

    set file_data [read $fp]
    close $fp

    set result [eval "dict create $file_data"]

    return $result
  }

  proc save_config { config_r } {
    variable imago_config_filename

    upvar $config_r config
    set fp [open $imago_config_filename w]

    puts $fp $config

    close $fp
  }

  proc set_config_value { config_r name value } {
    upvar $config_r config
    dict set config $name $value
  }

  proc get_config_value { config_r name } {
    upvar $config_r config

    if { [dict exists $config $name] } {
      set value [dict get $config $name]
    } else {
      set value ""
    }
    return $value
  }

  # found on http://wiki.tcl.tk/557
  proc invoke_browser {url} {
    # open is the OS X equivalent to xdg-open on Linux, start is used on Windows
    set commands {xdg-open open start}
    foreach browser $commands {
      if {$browser eq "start"} {
        set command [list {*}[auto_execok start] {}]
      } else {
        set command [auto_execok $browser]
      }
      if {[string length $command]} {
        break
      }
    }

    if {[string length $command] == 0} {
      return -code error "couldn't find browser"
    }
    if {[catch {exec {*}$command $url &} error]} {
      return -code error "couldn't execute '$command': $error"
    }
  }

  proc invoke_microsoft_access_macro { database_file_path macro_name } {

    puts "Invoking Microsoft Access macro ${database_file_path}:${macro_name}"

    # TODO: Get path to MSACCESS.EXE
    set command "C:/Program Files/Microsoft Office/root/Office16/MSACCESS.EXE"

    if {[string length $command] == 0} {
      return -code error "couldn't find Microsoft Access"
    }

    if {[catch {exec ${command} ${database_file_path} /X=${macro_name} &} error]} {
      return -code error "couldn't execute '$command': $error"
    }

  }

  #  Constants defined for mesh types
  variable CORE_TRAY_MESH "core_tray_mesh"
  variable LINEAR_MESH "linear_mesh"

  proc format_mesh_type { mesh_type } {

    if { $mesh_type == $::Imago::CORE_TRAY_MESH } {
      return "Core Tray Mesh"
    } elseif { $mesh_type == $::Imago::LINEAR_MESH } {
      return "Linear Mesh"
    } else {
      error "Unknown mesh type: $mesh_type"
    }

  }

  proc confirm_mesh_type { mesh_type_r { title "Select a mesh type:" } } {
    upvar $mesh_type_r mesh_type

    set imago_config [::Imago::load_config]
    set core_tray_mesh_folder_path [::Imago::get_config_value imago_config "core_tray_mesh_folder_path"]
    set linear_mesh_folder_path [::Imago::get_config_value imago_config "linear_mesh_folder_path"]

    if { [string length [string trim $core_tray_mesh_folder_path]] == 0 && [string length [string trim $linear_mesh_folder_path]] != 0 } {
      set mesh_type $::Imago::LINEAR_MESH
      return 1
    } elseif { [string length [string trim $core_tray_mesh_folder_path]] != 0 && [string length [string trim $linear_mesh_folder_path]] == 0 } {
      set mesh_type $::Imago::CORE_TRAY_MESH
      return 1
    } elseif { [string length [string trim $core_tray_mesh_folder_path]] == 0 && [string length [string trim $linear_mesh_folder_path]] == 0 } {
      display_warning "Invalid Configuration" "A Core Tray Mesh Folder or Linear Mesh Folder must be defined. Please update the configuration and try again."
      return 0
    } else {
      return [::do_display_select_mesh_type_form mesh_type $title]
    }
  }

  proc get_mesh_type_folder_path { mesh_type } {

    set imago_config [::Imago::load_config]

    if { $mesh_type == $::Imago::CORE_TRAY_MESH } {
      return [::Imago::get_config_value imago_config "core_tray_mesh_folder_path"]
    } elseif { $mesh_type == $::Imago::LINEAR_MESH } {
      return [::Imago::get_config_value imago_config "linear_mesh_folder_path"]
    } else {
      error "Unknown mesh type: $mesh_type"
    }

  }

  proc get_hole_mesh_file_path { mesh_type hole_id } {
    return [file join [get_mesh_type_folder_path $mesh_type] "${hole_id}.obj"]
  }

  proc mesh_exists { mesh_type hole_id } {
    return [file exists [get_hole_mesh_file_path $mesh_type $hole_id]]
  }
}

# Note: Guido demands that procedures are defined in the global namespace
# So we have these procedures here, and put a wrapper around it in the Imago namespace

proc ::do_display_warning { title message } {

  set form_definition {

    GuidoForm mesh_type_form {
      -default_buttons
      -label "@title@"
      -layout BoxLayout Y_AXIS
      -help_url SSI_PROFILES:imago/resource/scripts/readme.pdf

      GuidoPanel logoPanel1 {
        -layout FlowLayout Centre

        GuidoLabel ImageLabel {
          -icon "icons/warning-32.png"
          -label "<html>@message@</html>"
        }
      }
    }
  }

  set mappings_l [list]
  lappend mappings_l "@title@" $title
  lappend mappings_l "@message@" $message
  set form_definition [string map $mappings_l $form_definition]

  SclCreateGuidoForm form_h $form_definition {}

  $form_h SclRun {}

}

proc ::do_display_select_mesh_type_form { mesh_type_r { message "Select a mesh type:" } } {

  upvar $mesh_type_r mesh_type

  if { ![info exists mesh_type] } {
    set mesh_type $::Imago::CORE_TRAY_MESH
  }

  set mesh_type_form_definition {

    GuidoForm mesh_type_form {
      -default_buttons
      -label "Imago"
      -tip "Select a mesh type"
      -layout BoxLayout Y_AXIS
      -defaults_key IMAGO_SELECT_MESH_TYPE
      -help_url SSI_PROFILES:imago/resource/scripts/readme.pdf

      GuidoPanel title_panel {
        -layout FlowLayout Centre

        GuidoLabel title_label {
          -label "<html><h3>@message@</h3></html>"
          -font_highlight 1 1 1
        }
      }

      GuidoPanel panel {
        -layout FlowLayout Centre
        -border etched true
        -width 30

        GuidoButtonGroupPanel guido_mesh_type {
          -layout BoxLayout X_AXIS

          GuidoRadioButton core_tray_mesh {
            -caption "Core Tray"
            -selected_value "core_tray_mesh"
            -default true
          }

          GuidoRadioButton linear_mesh {
            -caption "Linear"
            -selected_value "linear_mesh"
          }
        }
      }
    }
  }

  set mappings_l [list]
  lappend mappings_l "@message@" $message
  set mesh_type_form_definition [string map $mappings_l $mesh_type_form_definition]

  SclCreateGuidoForm form_h $mesh_type_form_definition {}

  $form_h SclRun {}

  if { $_status == "cancel" } {
    return 0
  } else {
    set mesh_type $guido_mesh_type
    return 1
  }
}


